@@ -26,20 +26,10 @@
 #include <sys/stat.h>
 #include <errno.h>
 #include <libgen.h>
+#include <sys/types.h>
+#include <fcntl.h>
 
-typedef struct _FreeCallbackData FreeCallbackData;
-
-struct _MicrofeedWeakReference {
-	void* referenced;
-	unsigned int reference_count;
-	FreeCallbackData* free_callbacks;	
-};
-
-struct _FreeCallbackData {
-	FreeCallbackData* next;
-	MicrofeedFreeCallback free_callback;
-	void* user_data;
-};
+int microfeed_debug_memory = 0;
 
 void* microfeed_memory_allocate_bytes(size_t size) {
 	void* pointer;
@@ -54,6 +44,17 @@ void* microfeed_memory_allocate_bytes(size_t size) {
 	return pointer;
 }
 
+void* microfeed_memory_allocate_impl(size_t size, const char* name) {
+	void* pointer;
+	
+	pointer = microfeed_memory_allocate_bytes(size);
+	if (microfeed_debug_memory) {
+		printf("%p ALLOC   %s (%ld)\n", pointer, name, (long)size);
+	}
+	
+	return pointer;
+}
+
 /* This is not working and not used...
 void* microfeed_memory_allocate_with_strings_impl(size_t size, ...) {
 	void* pointer;
@@ -94,7 +95,11 @@ void* microfeed_memory_allocate_with_strings_impl(size_t size, ...) {
 }
 */
 
-void microfeed_memory_free(void* pointer) {	
+void microfeed_memory_free(void* pointer) {
+	if (microfeed_debug_memory) {
+		printf("%p FREE\n", pointer);
+	}
+
 	free(pointer);
 }
 
@@ -120,6 +125,30 @@ char* microfeed_util_string_concatenate(const char* s, ...) {
 	return result;	
 }
 
+char* microfeed_util_string_append(char* s, ...) {
+	char* result;
+	size_t size;
+	size_t length;
+	va_list argp;
+
+	size = strlen(s);
+	length = size + 1;
+	result = (char*)malloc(length);
+	memcpy(result, s, length);
+	va_start(argp, s);
+	while ((s = va_arg(argp, const char*))) {
+		length = strlen(s);
+		result = realloc(result, size + length + 1);
+		memcpy(result + size, s, length + 1);
+		size += length;
+	}
+	va_end(argp);
+	
+	free(s);
+
+	return result;	
+}
+
 char* microfeed_util_string_base64_encode(const char* s, size_t length) {
 	static const char base64[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
 	char* result;
@@ -254,80 +283,6 @@ char* microfeed_util_string_percent_encoding_unescape(const char* s) {
 	return result;
 }
 
-MicrofeedWeakReference* microfeed_weak_reference_new(void* referenced, MicrofeedWeakReference* existing_weak_reference) {
-	if (existing_weak_reference) {
-		existing_weak_reference->reference_count++;
-	} else {
-		existing_weak_reference = microfeed_memory_allocate(MicrofeedWeakReference);
-		existing_weak_reference->referenced = referenced;
-		existing_weak_reference->reference_count = 1;
-	}
-	
-	return existing_weak_reference;
-}
-
-void microfeed_weak_reference_free(MicrofeedWeakReference* weak_reference) {
-	weak_reference->reference_count--;
-	if (weak_reference->reference_count == 0 && !weak_reference->referenced) {
-		microfeed_memory_free(weak_reference);
-	}
-}
-
-void* microfeed_weak_reference_get_impl(MicrofeedWeakReference* weak_reference) {
-	
-	return weak_reference->referenced;
-}
-
-void microfeed_weak_reference_invalidate(MicrofeedWeakReference* weak_reference) {
-	FreeCallbackData* fcd;
-
-	if (weak_reference) {
-		for (fcd = weak_reference->free_callbacks; fcd; fcd = fcd->next) {
-			fcd->free_callback(weak_reference->referenced, fcd->user_data);
-			weak_reference->reference_count--;
-		}
-
-		weak_reference->referenced = NULL;
-
-		if (weak_reference->reference_count == 0) {
-			microfeed_memory_free(weak_reference);
-		}
-	}
-}
-
-MicrofeedWeakReference* microfeed_weak_reference_add_free_callback(MicrofeedWeakReference* existing_weak_reference, void* referenced, MicrofeedFreeCallback free_callback, void* user_data) {
-	MicrofeedWeakReference* weak_reference;
-	FreeCallbackData* fcd;
-	
-	weak_reference = microfeed_weak_reference_new(referenced, existing_weak_reference);
-
-	fcd = microfeed_memory_allocate(FreeCallbackData);
-	fcd->free_callback = free_callback;
-	fcd->user_data = user_data;
-	fcd->next = weak_reference->free_callbacks;
-	weak_reference->free_callbacks = fcd;
-	
-	return weak_reference;
-}
-
-void microfeed_weak_reference_remove_free_callback(MicrofeedWeakReference* weak_reference, MicrofeedFreeCallback free_callback, void* user_data) {
-	FreeCallbackData* fcd;
-	FreeCallbackData* previous = NULL;
-	
-	for (fcd = weak_reference->free_callbacks; fcd; fcd = fcd->next) {
-		if (fcd->free_callback == free_callback && fcd->user_data == user_data) {
-			if (previous) {
-				previous->next = fcd->next;
-			} else {
-				weak_reference->free_callbacks = fcd->next;
-			}
-			microfeed_memory_free(fcd);
-			break;
-		}
-		previous = fcd;
-	}
-}
-
 int microfeed_util_create_directory_recursively(const char* directory) {
 	int retval = 0;
 	char* s1;
@@ -349,3 +304,48 @@ int microfeed_util_create_directory_recursively(const char* directory) {
 	
 	return retval;
 }
+
+
+int microfeed_util_file_copy(const char* source_path, const char* destination_path) {
+	int retval = 0;
+	int source;
+	int destination;
+	struct stat stat_buffer;
+	char* buffer;
+	ssize_t left;
+	ssize_t wrote;
+	size_t position;
+	
+	if ((source = open(source_path, O_RDONLY)) >= 0) {
+		if ((destination = open(destination_path, O_WRONLY | O_CREAT | O_EXCL, 0666))) {
+			if (fstat(destination, &stat_buffer) == 0) {
+				buffer = alloca(stat_buffer.st_blksize);
+				while ((left = read(source, buffer, stat_buffer.st_blksize)) > 0) {
+					position = 0;
+					while ((wrote = write(destination, buffer + position, left)) != left && wrote >= 0) {
+						position += wrote;
+						left -= wrote;
+					}
+					if (wrote < 0) {
+						break;
+					}
+				}
+				if (left == 0 && wrote >= 0) {
+					retval = 1;
+				}
+			}
+			close(destination);
+			if (retval == 0) {
+				unlink(destination_path);
+			}
+		}
+		close(source);
+	}
+	
+	return retval;
+}
+
+void microfeed_assert_failed(const char* text) {
+	fprintf(stderr, "\n*****\nAssertion failed: %s\n\n*****\n\n", text);
+	abort();
+}
