@@ -52,11 +52,14 @@ typedef void (*MicrofeedStoreForeachFunction)(void* data, void* user_data);
 typedef int (*MicrofeedStoreCompareDatasFunction)(const void* data1, const void* data2, unsigned int index1, unsigned int index2, void* user_data);
 typedef void (*MicrofeedStoreFreeDataFunction)(void* data);
 
-MicrofeedStore* microfeed_store_new_sorted(MicrofeedStoreCompareKeysFunction compare_keys, MicrofeedStoreGetKeyFunction get_key);
-MicrofeedStore* microfeed_store_new_unsorted(MicrofeedStoreCompareKeysFunction compare_keys, MicrofeedStoreGetKeyFunction get_key);
+MicrofeedStore* microfeed_store_new_sorted_data(MicrofeedStoreCompareKeysFunction compare_keys, MicrofeedStoreGetKeyFunction get_key);
+MicrofeedStore* microfeed_store_new_unsorted_data(MicrofeedStoreCompareKeysFunction compare_keys, MicrofeedStoreGetKeyFunction get_key);
+MicrofeedStore* microfeed_store_new_sorted_weak_references(MicrofeedStoreCompareKeysFunction compare_keys, MicrofeedStoreGetKeyFunction get_key);
+MicrofeedStore* microfeed_store_new_unsorted_weak_references(MicrofeedStoreCompareKeysFunction compare_keys, MicrofeedStoreGetKeyFunction get_key);
 void microfeed_store_free(MicrofeedStore* store);
 
 int microfeed_store_is_sorted(MicrofeedStore* store);
+int microfeed_store_has_weak_references(MicrofeedStore* store);
 void microfeed_store_foreach(MicrofeedStore* store, MicrofeedStoreForeachFunction foreach, void* user_data);
 #define microfeed_store_get(s, k, t) ((t*)microfeed_store_get_impl(s, k))
 void* microfeed_store_get_impl(MicrofeedStore* store, const void* key);
@@ -74,6 +77,7 @@ void* microfeed_store_remove_index_impl(MicrofeedStore* store, unsigned int inde
 void microfeed_store_remove_and_free_all(MicrofeedStore* store, MicrofeedStoreFreeDataFunction free_data);
 MicrofeedStoreIterator* microfeed_store_iterate(MicrofeedStore* store, const void* start_key);
 void microfeed_store_sort(MicrofeedStore* store, MicrofeedStoreCompareDatasFunction compare_datas, void* user_data);
+void microfeed_store_clean_weak_references(MicrofeedStore* store);
 
 void microfeed_store_iterator_free(MicrofeedStoreIterator* iterator);
 void* microfeed_store_iterator_get_impl(MicrofeedStoreIterator* iterator);
