@@ -24,6 +24,8 @@
 #include <pthread.h>
 #include <signal.h>
 
+#include <stdio.h>
+
 struct _MicrofeedThread {
 	MicrofeedThread* next;
 	MicrofeedThread* previous;
@@ -54,7 +56,7 @@ struct _MicrofeedMutex {
 	void* mutex_implementation;
 };
 
-static void* default_thread_new(MicrofeedThreadFunction function, void* data);
+static void* default_thread_new(void* (*function)(void* data), void* data);
 static void default_thread_free(void* thread_implementation);
 static void* default_thread_get_current(void);
 static void default_thread_send_signal(void* thread_implementation, int signal_number);
@@ -115,8 +117,8 @@ void microfeed_thread_cleanup(void) {
 		}
 		if (thread) {
 			microfeed_mutex_unlock(threads_mutex);
-			
-			microfeed_thread_join(thread);
+
+/* TODO: What to do (if anything) if a thread is still running? */
 			
 			microfeed_mutex_lock(threads_mutex);
 		}
@@ -165,8 +167,6 @@ MicrofeedThread* microfeed_thread_new_with_exit_callback(MicrofeedThreadFunction
 
 void microfeed_thread_free(MicrofeedThread* thread) {
 	if (thread) {
-		functions.thread_free(thread->thread_implementation);
-	
 		microfeed_mutex_lock(threads_mutex);
 	
 		if (thread->previous) {
@@ -180,6 +180,7 @@ void microfeed_thread_free(MicrofeedThread* thread) {
 	
 		microfeed_mutex_unlock(threads_mutex);
 	
+		microfeed_mutex_free(thread->mutex);
 		microfeed_memory_free(thread);
 	}
 }
@@ -242,11 +243,9 @@ MicrofeedThread* microfeed_thread_get_current(void) {
 }
 
 void microfeed_thread_send_signal(MicrofeedThread* thread, int signal_number) {
-	functions.thread_send_signal(thread->thread_implementation, signal_number);
-}
-
-void microfeed_thread_join(MicrofeedThread* thread) {
-	functions.thread_join(thread->thread_implementation);
+	if (thread->thread_implementation) {
+		functions.thread_send_signal(thread->thread_implementation, signal_number);
+	}
 }
 
 unsigned long microfeed_thread_get_id(MicrofeedThread* thread) {
@@ -254,7 +253,6 @@ unsigned long microfeed_thread_get_id(MicrofeedThread* thread) {
 	return thread->id;	
 }
 
-
 MicrofeedThreadPool* microfeed_thread_pool_new(unsigned int maximum_thread_count) {
 
 	return microfeed_thread_pool_new_with_exit_callback(maximum_thread_count, NULL, NULL);
@@ -273,12 +271,20 @@ MicrofeedThreadPool* microfeed_thread_pool_new_with_exit_callback(unsigned int m
 	thread_pool->exit_callback = exit_callback;
 	thread_pool->user_data = user_data;
 	thread_pool->mutex = microfeed_mutex_new();
-	thread_pool->waiting_threads = microfeed_store_new_unsorted(microfeed_store_compare_keys_direct, microfeed_store_get_key_direct);
+	thread_pool->waiting_threads = microfeed_store_new_unsorted_data(microfeed_store_compare_keys_direct, microfeed_store_get_key_direct);
 	
 	return thread_pool;
 	
 }
 
+void microfeed_thread_pool_free(MicrofeedThreadPool* thread_pool) {
+	microfeed_assert(thread_pool->started_threads == 0);
+	
+	microfeed_mutex_free(thread_pool->mutex);	
+	microfeed_store_free(thread_pool->waiting_threads);
+	microfeed_memory_free(thread_pool);
+}
+
 MicrofeedThread* microfeed_thread_pool_queue_thread(MicrofeedThreadPool* thread_pool, MicrofeedThreadFunction function, void* data) {
 
 	return microfeed_thread_pool_queue_thread_with_exit_callback(thread_pool, function, data, NULL, NULL);
@@ -309,9 +315,8 @@ MicrofeedThread* microfeed_thread_pool_queue_thread_with_exit_callback(Microfeed
 			thread->next->previous = thread;
 		}
 		threads = thread;
-				
+			
 		microfeed_mutex_lock(thread->mutex);
-	
 		thread->thread_implementation = functions.thread_new(thread_function, thread);
 	
 		microfeed_mutex_unlock(thread->mutex);
@@ -341,6 +346,7 @@ void microfeed_thread_pool_set_maximum_thread_count(MicrofeedThreadPool* thread_
 	thread_pool->max_threads = maximum_thread_count;
 }
 
+
 MicrofeedMutex* microfeed_mutex_new(void) {
 	MicrofeedMutex* mutex;
 	
@@ -363,11 +369,17 @@ void microfeed_mutex_unlock(MicrofeedMutex* mutex) {
 	functions.mutex_unlock(mutex->mutex_implementation);
 }
 
-static void* default_thread_new(MicrofeedThreadFunction function, void* data) {
+static void* default_thread_new(void* (*function)(void* data), void* data) {
+	pthread_attr_t attr;
 	pthread_t* thread_implementation;
-	
+
+	pthread_attr_init(&attr);
+	pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);
+
 	thread_implementation = microfeed_memory_allocate(pthread_t);
-	pthread_create(thread_implementation, NULL, (void* (*)(void*))function, data);
+	pthread_create(thread_implementation, &attr, (void* (*)(void*))function, data);
+
+	pthread_attr_destroy(&attr);
 
 	return (void*)thread_implementation;
 }
@@ -384,7 +396,7 @@ static void* default_thread_get_current(void) {
 	self = pthread_self();
 	
 	for (thread = threads; thread; thread = thread->next) {
-		if (*((pthread_t*)thread->thread_implementation) == self) {
+		if (thread->thread_implementation && *((pthread_t*)thread->thread_implementation) == self) {
 			break;
 		}
 	}
@@ -421,7 +433,16 @@ static void default_mutex_free(void* mutex_implementation) {
 }
 
 static void default_mutex_lock(void* mutex_implementation) {
-	pthread_mutex_lock((pthread_mutex_t*)mutex_implementation);
+	struct timespec abs_timeout;
+	
+	/* pthread_mutex_lock((pthread_mutex_t*)mutex_implementation); */
+
+	abs_timeout.tv_sec = time(NULL) + 120;
+	abs_timeout.tv_nsec = 0;
+	if (pthread_mutex_timedlock((pthread_mutex_t*)mutex_implementation, &abs_timeout)) {
+		fprintf(stderr, "ERROR: Could not lock mutex: %p\n", mutex_implementation);
+		abort();
+	}
 }
 
 static void default_mutex_unlock(void* mutex_implementation) {
@@ -431,9 +452,16 @@ static void default_mutex_unlock(void* mutex_implementation) {
 static void* thread_function(void* data) {
 	MicrofeedThread* thread;
 	MicrofeedThreadPool* thread_pool;
+	void* thread_implementation;
 	
 	thread = (MicrofeedThread*)data;
+
+	microfeed_mutex_lock(thread->mutex);
+
 	thread_pool = thread->thread_pool;
+	thread_implementation = thread->thread_implementation;
+
+	microfeed_mutex_unlock(thread->mutex);
 
 	do {
 		thread->out_data = thread->function(thread->in_data);
@@ -448,8 +476,8 @@ static void* thread_function(void* data) {
 		if (thread->reference_count == 0) {
 			microfeed_thread_free(thread);
 		} else {
-		//	thread->thread_implementation = NULL;
-		
+			thread->thread_implementation = NULL;
+
 			microfeed_mutex_unlock(thread->mutex);
 		}
 
@@ -457,9 +485,9 @@ static void* thread_function(void* data) {
 		
 		if (thread_pool) {
 			microfeed_mutex_lock(thread_pool->mutex);
-
 			if (thread_pool->started_threads <= thread_pool->max_threads && microfeed_store_get_size(thread_pool->waiting_threads) > 0) {
-				thread = microfeed_store_remove_index(thread_pool->waiting_threads, 0, MicrofeedThread);	
+				thread = microfeed_store_remove_index(thread_pool->waiting_threads, 0, MicrofeedThread);
+				thread->thread_implementation = thread_implementation;
 			} else {
 				thread_pool->started_threads--;
 				if (thread_pool->exit_callback) {
@@ -470,6 +498,8 @@ static void* thread_function(void* data) {
 			microfeed_mutex_unlock(thread_pool->mutex);
 		}
 	} while (thread);
+	
+	functions.thread_free(thread_implementation);
 
 	return NULL;	
 }
