@@ -20,22 +20,28 @@
 #include <microfeed-common/microfeedmisc.h>
 #include <microfeed-common/microfeedthread.h>
 #include <microfeed-common/microfeedstore.h>
+#include <microfeed-common/microfeedobject.h>
+#include <microfeed-common/microfeedprotocol.h>
 
 #include <errno.h>
 #include <string.h>
 #include <db.h>
+#include <unistd.h>
+#include <fcntl.h>
+#include <sys/types.h>
+#include <sys/stat.h>
 
 struct _MicrofeedDatabaseEnvironment {
-	unsigned int reference_count;
-	MicrofeedMutex* mutex;
-	char* name;
+	MicrofeedObject object;
+	char* public_name;
+	char* private_name;
+	char* directory;
 	DB_ENV* db_env;
 	MicrofeedStore* databases;
 };
 
 struct _MicrofeedDatabase {
-	unsigned int reference_count;
-	MicrofeedMutex* mutex;
+	MicrofeedObject object;
 	MicrofeedDatabaseEnvironment* database_environment;
 	char* name;
 	MicrofeedDatabaseCompareFunction compare_function;
@@ -51,6 +57,7 @@ struct _MicrofeedDatabaseIndex {
 	char* name;
 	DB* db;
 	MicrofeedDatabaseIndexFunction index_function;
+	MicrofeedDatabaseCompareFunction compare_function;
 };
 
 struct _MicrofeedDatabaseIterator {
@@ -67,9 +74,13 @@ struct _MicrofeedDatabaseIterator {
 	int backwards : 1;
 };
 
+static void database_environment_free_function(MicrofeedDatabaseEnvironment* database_environment);
+static void database_free_function(MicrofeedDatabase* database);
+
 static void lock_environment_for_writing(MicrofeedDatabaseEnvironment* database_environment);
 static void unlock_environment(MicrofeedDatabaseEnvironment* database_environment);
 static int key_compare_function(DB *db, const DBT *dbt1, const DBT *dbt2);
+static int dup_compare_function(DB *db, const DBT *dbt1, const DBT *dbt2);
 static int index_callback(DB* secondary, const DBT* key, const DBT* data, DBT* result);
 static void remove_data(MicrofeedDatabase* database, DB* db, const void* key, const size_t key_size);
 static void remove_data_range(MicrofeedDatabase* database, DB* db, const void* start_key, const size_t start_key_size, const void* end_key, const size_t end_key_size);
@@ -77,93 +88,140 @@ static void iterator_free(MicrofeedDatabaseIterator* iterator);
 static void database_iterator_next(MicrofeedDatabaseIterator* iterator);
 static void database_index_iterator_next(MicrofeedDatabaseIterator* iterator);
 	
+static MicrofeedClass microfeed_database_environment_class = {
+	"MicrofeedDatabaseEnvironment",
+	(MicrofeedObjectFreeFunction)database_environment_free_function
+};
+
+static MicrofeedClass microfeed_database_class = {
+	"MicrofeedDatabase",
+	(MicrofeedObjectFreeFunction)database_free_function
+};
+	
+static MicrofeedDatabaseEnvironment* new_database_environment(const char* public_name, const char* private_name, const char* directory) {
+	MicrofeedDatabaseEnvironment* database_environment = NULL;
+	DB_ENV* db_env;
+	int error;
+
+	if (!(error = db_env_create(&db_env, 0)) &&
+	    !(error = db_env->open(db_env, directory, DB_CREATE | DB_THREAD | DB_PRIVATE | DB_INIT_CDB | DB_INIT_MPOOL, 0))) {
+		database_environment = microfeed_object_new(&microfeed_database_environment_class, MicrofeedDatabaseEnvironment);
+		database_environment->public_name = strdup(public_name);
+		database_environment->private_name = strdup(private_name);
+		database_environment->directory = strdup(directory);
+		database_environment->db_env = db_env;
+		database_environment->databases = microfeed_store_new_sorted_weak_references((MicrofeedStoreCompareKeysFunction)strcmp,
+			                                                	             (MicrofeedStoreGetKeyFunction)microfeed_database_get_name);
+	} else {
+		fprintf(stderr, "ERROR: Failed to create database environment: %s\n", db_strerror(error));
+	}
+	
+	return database_environment;
+
+}
+
 MicrofeedDatabaseEnvironment* microfeed_database_environment_new(const char* name, const char* directory) {
 	MicrofeedDatabaseEnvironment* database_environment = NULL;
+	char buffer[1024];
+	char* source_path;
+	char* destination_path;
+	int fd;
+	struct flock lock;
 
+	snprintf(buffer, 1024, "%s%c%ld", name, MICROFEED_PUBLISHER_IDENTIFIER_SEPARATOR_CHAR, (long)getpid());
 	if (microfeed_util_create_directory_recursively(directory)) {
-		database_environment = microfeed_memory_allocate(MicrofeedDatabaseEnvironment);
-		microfeed_util_create_directory_recursively(directory);
-		if (db_env_create(&database_environment->db_env, 0) || database_environment->db_env->open(database_environment->db_env, directory, DB_CREATE | DB_THREAD | DB_PRIVATE | DB_INIT_CDB | DB_INIT_MPOOL, 0)) {
-			microfeed_memory_free(database_environment);
-			database_environment = NULL;
+		source_path = microfeed_util_string_concatenate(directory, "/", name, NULL);
+		destination_path = microfeed_util_string_concatenate(directory, "/", buffer, NULL);
+		if ((fd = open(source_path, O_RDWR)) >= 0) {
+			lock.l_type = F_RDLCK;
+			lock.l_whence = SEEK_SET;
+			lock.l_start = 0;
+			lock.l_len = 0;
+			if (fcntl(fd, F_GETLK, &lock) == 0) {
+				/* Note: file_copy opens and closes the source file, and thus, releases the lock. */
+				if (microfeed_util_file_copy(source_path, destination_path)) {
+					if (!(database_environment = new_database_environment(name, buffer, directory))) {
+						unlink(destination_path);
+					}
+				}
+			}
+			/* This, at least, releases the lock. */
+			close(fd);
 		} else {
-			database_environment->reference_count = 1;
-			database_environment->mutex = microfeed_mutex_new();
-			database_environment->name = strdup(name);
-			database_environment->databases = microfeed_store_new_sorted((MicrofeedStoreCompareKeysFunction)strcmp,
-			                                                	  (MicrofeedStoreGetKeyFunction)microfeed_database_get_name);	
+			database_environment = new_database_environment(name, buffer, directory);
 		}
+		microfeed_memory_free(source_path);
+		microfeed_memory_free(destination_path);
 	}
 
 	return database_environment;
 }
 
-void microfeed_database_environment_free(MicrofeedDatabaseEnvironment* database_environment) {
-	free(database_environment->name);
-	database_environment->db_env->close(database_environment->db_env, 0);
-	database_environment->db_env = NULL;
-	microfeed_memory_free(database_environment);
-}
-
-MicrofeedDatabaseEnvironment* microfeed_database_environment_ref(MicrofeedDatabaseEnvironment* database_environment) {
-	microfeed_mutex_lock(database_environment->mutex);
-
-	database_environment->reference_count++;
-
-	microfeed_mutex_unlock(database_environment->mutex);
+static void database_environment_free_function(MicrofeedDatabaseEnvironment* database_environment) {
+	char* source_path;
+	char* destination_path;
+	int error;
 	
-	return database_environment;
-}
+	microfeed_store_clean_weak_references(database_environment->databases);
+	microfeed_assert(microfeed_store_get_size(database_environment->databases) == 0);
 
-void microfeed_database_environment_unref(MicrofeedDatabaseEnvironment* database_environment) {
-	microfeed_mutex_lock(database_environment->mutex);
-
-	database_environment->reference_count--;
-	if (database_environment->reference_count == 0) {
-		microfeed_database_environment_free(database_environment);
+	if (!(error = database_environment->db_env->close(database_environment->db_env, 0))) {
+		source_path = microfeed_util_string_concatenate(database_environment->directory, "/", database_environment->private_name, NULL);
+		destination_path = microfeed_util_string_concatenate(database_environment->directory, "/", database_environment->public_name, NULL);
+		unlink(destination_path);
+		link(source_path, destination_path);
+		unlink(source_path);
+		microfeed_memory_free(source_path);
+		microfeed_memory_free(destination_path);
 	} else {
+		fprintf(stderr, "ERROR: Could not close database environment: %s\n", db_strerror(error));
+	}
 
-		microfeed_mutex_unlock(database_environment->mutex);
-	}		
+	free(database_environment->public_name);
+	free(database_environment->private_name);
+	free(database_environment->directory);
+	microfeed_store_free(database_environment->databases);
+	database_environment->db_env = NULL;
 }
-	
-MicrofeedDatabase* microfeed_database_environment_get_database(MicrofeedDatabaseEnvironment* database_environment, const char* name, MicrofeedDatabaseCompareFunction compare_function) {
-	MicrofeedDatabase* database;
+
+MicrofeedDatabase* microfeed_database_environment_get_database(MicrofeedDatabaseEnvironment* database_environment, const char* name, MicrofeedDatabaseCompareFunction compare_function, int create) {
+	MicrofeedDatabase* database = NULL;
+	DB* db;
+	int error;
 	
 	lock_environment_for_writing(database_environment);
 
-	if ((database = microfeed_store_get(database_environment->databases, name, MicrofeedDatabase))) {
-		microfeed_database_ref(database);
-	} else {
-		database = microfeed_memory_allocate(MicrofeedDatabase);
-		if (db_create(&database->db, database_environment->db_env, 0) ||
-		    (compare_function && database->db->set_bt_compare(database->db, key_compare_function)) ||
-		    database->db->open(database->db, NULL, database_environment->name, name, DB_BTREE, DB_CREATE | DB_THREAD, 0)) {
-			microfeed_memory_free(database);
-			database = NULL;
-		} else {
-			database->reference_count = 1;
-			database->mutex = microfeed_mutex_new();
-			database->database_environment = database_environment;
-			database_environment->reference_count++;
+	if (!(database = microfeed_store_get(database_environment->databases, name, MicrofeedDatabase)) &&
+	    !(error = db_create(&db, database_environment->db_env, 0))) {
+	    	if ((!compare_function || !(error = db->set_bt_compare(db, key_compare_function))) &&
+		    !(error = db->open(db, NULL, database_environment->private_name, name, DB_BTREE, DB_THREAD | (create ? DB_CREATE : 0), 0))) {
+			database = microfeed_object_new(&microfeed_database_class, MicrofeedDatabase);
+			database->database_environment = microfeed_object_ref(database_environment, MicrofeedDatabaseEnvironment);
 			database->name = strdup(name);
 			database->compare_function = compare_function;
+			database->db = db;
 			database->indices = NULL;
 			database->iterators = NULL;
 			database->db->app_private = database;
-				
+
 			microfeed_store_insert(database_environment->databases, database);
+		} else {
+			if ((error = db->close(db, 0))) {
+				fprintf(stderr, "ERROR: Could not closed database that is not opened: %s\n", db_strerror(error));
+			}
 		}
 	}
 
 	unlock_environment(database_environment);
-	
+
 	return database;
 }
 
-MicrofeedDatabaseIndex* microfeed_database_get_index(MicrofeedDatabase* database, const char* name, MicrofeedDatabaseIndexFunction index_function) {
+MicrofeedDatabaseIndex* microfeed_database_get_index(MicrofeedDatabase* database, const char* name, MicrofeedDatabaseIndexFunction index_function, MicrofeedDatabaseCompareFunction compare_function) {
 	MicrofeedDatabaseIndex* database_index;
 	char* full_name;
+	DB* db;
+	int error;
 	
 	lock_environment_for_writing(database->database_environment);
 
@@ -173,24 +231,27 @@ MicrofeedDatabaseIndex* microfeed_database_get_index(MicrofeedDatabase* database
 		}
 	}
 	if (!database_index) {
-		database_index = microfeed_memory_allocate(MicrofeedDatabaseIndex);
 		full_name = microfeed_util_string_concatenate(database->name, ":microfeed.index:", name, NULL);
-		if (db_create(&database_index->db, database->database_environment->db_env, 0) ||
-		    database_index->db->open(database_index->db, NULL, database->database_environment->name, full_name, DB_BTREE, DB_CREATE | DB_THREAD, 0)) {
-			microfeed_memory_free(database_index);
-			database_index = NULL;
-		} else if (database->db->associate(database->db, NULL, database_index->db, index_callback, DB_CREATE)) { // | DB_IMMUTABLE_KEY)) {
-			database_index->db->close(database_index->db, 0);
-			microfeed_memory_free(database_index);
-			database_index = NULL;
+		if (!(error = db_create(&db, database->database_environment->db_env, 0)) &&
+		    (!compare_function || !(error = db->set_dup_compare(db, dup_compare_function))) &&
+		    !(error = db->open(db, NULL, database->database_environment->private_name, full_name, DB_BTREE, DB_CREATE | DB_THREAD, 0))) {
+			if (!(error = db->associate(database->db, NULL, db, index_callback, DB_CREATE))) { // | DB_IMMUTABLE_KEY)) {
+				database_index = microfeed_memory_allocate(MicrofeedDatabaseIndex);
+				database_index->db = db;
+				database_index->database = database;
+				database_index->name = strdup(name);
+				database_index->index_function = index_function;
+				database_index->compare_function = compare_function;		
+				database_index->db->app_private = database_index;
+
+				database_index->next = database->indices;
+				database->indices = database_index;
+			} else {
+				fprintf(stderr, "ERROR: Failed to associate secondary index: %s\n", db_strerror(error));
+				db->close(db, 0);
+			}
 		} else {
-			database_index->database = database;
-			database_index->name = strdup(name);
-			database_index->index_function = index_function;			
-			database_index->db->app_private = database_index;
-				
-			database_index->next = database->indices;
-			database->indices = database_index;
+			fprintf(stderr, "ERROR: Failed to create index database: %s\n", db_strerror(error));
 		}
 		free(full_name);
 	}
@@ -200,55 +261,30 @@ MicrofeedDatabaseIndex* microfeed_database_get_index(MicrofeedDatabase* database
 	return database_index;	
 }
 
-void microfeed_database_free(MicrofeedDatabase* database) {
+static void database_free_function(MicrofeedDatabase* database) {
 	MicrofeedDatabaseIndex* database_index;
 	MicrofeedDatabaseIndex* next_database_index;
-	
+	int error;
+
+	microfeed_assert(!database->iterators);
+
 	for (database_index = database->indices; database_index; database_index = next_database_index) {
 		next_database_index = database_index->next;
-		database_index->db->close(database_index->db, 0);
+		if ((error = database_index->db->close(database_index->db, 0))) {
+			fprintf(stderr, "ERROR: Failed to close database index: %s\n", database_index->name);
+		}
 		free(database_index->name);
 		microfeed_memory_free(database_index);
 	}	
-	database->db->close(database->db, 0);
+	if ((error = database->db->close(database->db, 0))) {
+		fprintf(stderr, "ERROR: Failed to close database: %s\n", database->name);
+	}
 	database->db = NULL;
 
-	microfeed_mutex_lock(database->database_environment->mutex);
-
-	microfeed_store_remove(database->database_environment->databases, database);
-
-	microfeed_mutex_unlock(database->database_environment->mutex);
-
-	microfeed_mutex_free(database->mutex);
-	database->mutex = NULL;
-	microfeed_database_environment_unref(database->database_environment);
+	microfeed_object_unref(database->database_environment, MicrofeedDatabaseEnvironment);
 	database->database_environment = NULL;
 	free(database->name);
 	database->name = NULL;
-		
-	microfeed_memory_free(database);
-}
-
-MicrofeedDatabase* microfeed_database_ref(MicrofeedDatabase* database) {
-	microfeed_mutex_lock(database->mutex);
-	
-	database->reference_count++;
-	
-	microfeed_mutex_unlock(database->mutex);
-	
-	return database;
-}
-
-void microfeed_database_unref(MicrofeedDatabase* database) {
-	microfeed_mutex_lock(database->mutex);
-	
-	database->reference_count--;
-	if (database->reference_count == 0) {
-		microfeed_database_free(database);
-	} else {
-	
-		microfeed_mutex_unlock(database->mutex);
-	}
 }
 
 const char* microfeed_database_get_name(MicrofeedDatabase* database) {
@@ -261,7 +297,7 @@ int microfeed_database_get_data(MicrofeedDatabase* database, const void* key, si
 	DBT dbt_key;
 	DBT dbt_data;
 	
-	microfeed_mutex_lock(database->mutex);
+	microfeed_object_lock(database, MicrofeedDatabase);
 
 	memset(&dbt_key, 0, sizeof(DBT));
 	memset(&dbt_data, 0, sizeof(DBT));
@@ -275,7 +311,7 @@ int microfeed_database_get_data(MicrofeedDatabase* database, const void* key, si
 		retvalue = 1;
 	}
 	
-	microfeed_mutex_unlock(database->mutex);
+	microfeed_object_unlock(database, MicrofeedDatabase);
 	
 	return retvalue;
 }
@@ -285,7 +321,7 @@ int microfeed_database_get_data_partial(MicrofeedDatabase* database, const void*
 	DBT dbt_key;
 	DBT dbt_data;
 	
-	microfeed_mutex_lock(database->mutex);
+	microfeed_object_lock(database, MicrofeedDatabase);
 
 	memset(&dbt_key, 0, sizeof(DBT));
 	memset(&dbt_data, 0, sizeof(DBT));
@@ -302,7 +338,7 @@ int microfeed_database_get_data_partial(MicrofeedDatabase* database, const void*
 		retvalue = 1;
 	}
 	
-	microfeed_mutex_unlock(database->mutex);
+	microfeed_object_unlock(database, MicrofeedDatabase);
 	
 	return retvalue;
 }
@@ -311,7 +347,7 @@ int microfeed_database_get_data_partial(MicrofeedDatabase* database, const void*
 MicrofeedDatabaseIterator* microfeed_database_iterate(MicrofeedDatabase* database, const void* start_key, const size_t start_key_size, int backwards) {
 	MicrofeedDatabaseIterator* iterator;
 
-	microfeed_mutex_lock(database->mutex);
+	microfeed_object_lock(database, MicrofeedDatabase);
 
 	iterator = microfeed_memory_allocate(MicrofeedDatabaseIterator);
 	iterator->database = database;
@@ -341,7 +377,7 @@ MicrofeedDatabaseIterator* microfeed_database_iterate(MicrofeedDatabase* databas
 	iterator->previous = NULL;
 	database->iterators = iterator;
 
-	microfeed_mutex_unlock(database->mutex);
+	microfeed_object_unlock(database, MicrofeedDatabase);
 
 	return iterator;
 }
@@ -379,8 +415,6 @@ void microfeed_database_replace_data(MicrofeedDatabase* database, const void* ke
 		fprintf(stderr, "DB->put failed: %s\n", db_strerror(error));
 	}
 
-	database->db->sync(database->db, 0);
-
 	unlock_environment(database->database_environment);
 }
 
@@ -404,8 +438,6 @@ void microfeed_database_replace_data_partial(MicrofeedDatabase* database, const
 		fprintf(stderr, "DB->put failed in replace_data_partial: %s\n", db_strerror(error));
 	}
 
-	database->db->sync(database->db, 0);
-
 	unlock_environment(database->database_environment);
 }
 
@@ -433,7 +465,7 @@ int microfeed_database_index_get_data(MicrofeedDatabaseIndex* database_index, co
 	DBT dbt_key;
 	DBT dbt_data;
 
-	microfeed_mutex_lock(database_index->database->mutex);
+	microfeed_object_lock(database_index->database, MicrofeedDatabase);
 
 	memset(&dbt_index_key, 0, sizeof(DBT));
 	memset(&dbt_key, 0, sizeof(DBT));
@@ -451,7 +483,7 @@ int microfeed_database_index_get_data(MicrofeedDatabaseIndex* database_index, co
 		retvalue = 1;
 	}
 	
-	microfeed_mutex_unlock(database_index->database->mutex);
+	microfeed_object_unlock(database_index->database, MicrofeedDatabase);
 	
 	return retvalue;
 	
@@ -460,7 +492,7 @@ int microfeed_database_index_get_data(MicrofeedDatabaseIndex* database_index, co
 MicrofeedDatabaseIterator* microfeed_database_index_iterate(MicrofeedDatabaseIndex* database_index, const void* start_key, const size_t start_key_size, int backwards) {
 	MicrofeedDatabaseIterator* iterator;
 
-	microfeed_mutex_lock(database_index->database->mutex);
+	microfeed_object_lock(database_index->database, MicrofeedDatabase);
 
 	iterator = microfeed_memory_allocate(MicrofeedDatabaseIterator);
 	iterator->database = database_index->database;
@@ -492,7 +524,7 @@ MicrofeedDatabaseIterator* microfeed_database_index_iterate(MicrofeedDatabaseInd
 	iterator->previous = NULL;
 	database_index->database->iterators = iterator;
 
-	microfeed_mutex_unlock(database_index->database->mutex);
+	microfeed_object_unlock(database_index->database, MicrofeedDatabase);
 
 	return iterator;
 }
@@ -506,7 +538,7 @@ void microfeed_database_index_remove_data_range(MicrofeedDatabaseIndex* database
 }
 
 void microfeed_database_iterator_free(MicrofeedDatabaseIterator* iterator) {
-	microfeed_mutex_lock(iterator->database->mutex);
+	microfeed_object_lock(iterator->database, MicrofeedDatabase);
 
 	if (iterator->next) {
 		iterator->next->previous = iterator->previous;
@@ -517,7 +549,7 @@ void microfeed_database_iterator_free(MicrofeedDatabaseIterator* iterator) {
 		iterator->database->iterators = iterator->next;
 	}
 
-	microfeed_mutex_unlock(iterator->database->mutex);
+	microfeed_object_unlock(iterator->database, MicrofeedDatabase);
 
 	iterator_free(iterator);
 	iterator->database = NULL;
@@ -527,7 +559,7 @@ void microfeed_database_iterator_free(MicrofeedDatabaseIterator* iterator) {
 int microfeed_database_iterator_get(MicrofeedDatabaseIterator* iterator, const void** key, size_t* key_size, const void** data, size_t* data_size) {
 	int success = 0;
 
-	microfeed_mutex_lock(iterator->database->mutex);
+	microfeed_object_lock(iterator->database, MicrofeedDatabase);
 	
 	if (iterator->data.data) {
 		*key = iterator->key.data;
@@ -537,21 +569,22 @@ int microfeed_database_iterator_get(MicrofeedDatabaseIterator* iterator, const v
 		success = 1;
 	}
 	
-	microfeed_mutex_unlock(iterator->database->mutex);
+	microfeed_object_unlock(iterator->database, MicrofeedDatabase);
 
 	return success;
 }
 
 void microfeed_database_iterator_next(MicrofeedDatabaseIterator* iterator) {
-	microfeed_mutex_lock(iterator->database->mutex);
+	microfeed_object_lock(iterator->database, MicrofeedDatabase);
 
 	iterator->next_function(iterator);
 	
-	microfeed_mutex_unlock(iterator->database->mutex);
+	microfeed_object_unlock(iterator->database, MicrofeedDatabase);
 }
 
 static void remove_data(MicrofeedDatabase* database, DB* db, const void* key, const size_t key_size) {
 	DBT dbt_key;
+	int error;
 	
 	lock_environment_for_writing(database->database_environment);
 
@@ -559,7 +592,9 @@ static void remove_data(MicrofeedDatabase* database, DB* db, const void* key, co
 	dbt_key.data = (void*)key;
 	dbt_key.size = key_size;
 
-	db->del(db, NULL, &dbt_key, 0);	
+	if ((error = db->del(db, NULL, &dbt_key, 0))) {
+		fprintf(stderr, "ERROR: Failed to delete data from database: %s\n", db_strerror(error));
+	}
 
 	unlock_environment(database->database_environment);
 }
@@ -590,6 +625,8 @@ static void remove_data_range(MicrofeedDatabase* database, DB* db, const void* s
 			} while (!db_cursor->c_get(db_cursor, &dbt_key, &dbt_data, DB_NEXT));
 		}
 		db_cursor->c_close(db_cursor);
+		free(dbt_key.data);
+		free(dbt_data.data);
 	}
 
 	unlock_environment(database->database_environment);
@@ -670,12 +707,14 @@ static void lock_environment_for_writing(MicrofeedDatabaseEnvironment* database_
 	MicrofeedDatabase* database;
 	MicrofeedDatabaseIterator* database_iterator;
 
-	microfeed_mutex_lock(database_environment->mutex);
+	microfeed_object_lock(database_environment, MicrofeedDatabaseEnvironment);
 
+	/* TODO: Must be cleaned before since microfeed_store_iterator_get has a bug! */
+	microfeed_store_clean_weak_references(database_environment->databases);
 	for (store_iterator = microfeed_store_iterate(database_environment->databases, NULL);
 	     (database = microfeed_store_iterator_get(store_iterator, MicrofeedDatabase));
 	     microfeed_store_iterator_next(store_iterator)) {
-		microfeed_mutex_lock(database->mutex);
+		microfeed_object_lock(database, MicrofeedDatabase);
 		for (database_iterator = database->iterators; database_iterator; database_iterator = database_iterator->next) {
 			if (database_iterator->data.data && database_iterator->db_cursor) {
 				database_iterator->db_cursor->c_close(database_iterator->db_cursor);
@@ -683,19 +722,23 @@ static void lock_environment_for_writing(MicrofeedDatabaseEnvironment* database_
 			}
 		}
 	}
+	microfeed_store_iterator_free(store_iterator);
 }
 
 static void unlock_environment(MicrofeedDatabaseEnvironment* database_environment) {
 	MicrofeedStoreIterator* store_iterator;
 	MicrofeedDatabase* database;
 
+	/* TODO: Must be cleaned before since microfeed_store_iterator_get has a bug! */
+	microfeed_store_clean_weak_references(database_environment->databases);
 	for (store_iterator = microfeed_store_iterate(database_environment->databases, NULL);
 	     (database = microfeed_store_iterator_get(store_iterator, MicrofeedDatabase));
 	     microfeed_store_iterator_next(store_iterator)) {
-	    microfeed_mutex_unlock(database->mutex);
+		microfeed_object_unlock(database, MicrofeedDatabase);
 	}
+	microfeed_store_iterator_free(store_iterator);
 
-	microfeed_mutex_unlock(database_environment->mutex);
+	microfeed_object_unlock(database_environment, MicrofeedDatabaseEnvironment);
 }
 
 static int index_callback(DB* db, const DBT* key, const DBT* data, DBT* result) {
@@ -724,3 +767,14 @@ static int key_compare_function(DB *db, const DBT *dbt1, const DBT *dbt2) {
 
 	return retvalue;
 }
+
+static int dup_compare_function(DB *db, const DBT *dbt1, const DBT *dbt2) {
+	int retvalue = 0;
+	MicrofeedDatabaseIndex* database_index;
+	
+	if ((database_index = (MicrofeedDatabaseIndex*)db->app_private)) {
+		retvalue = database_index->compare_function(dbt1->data, dbt1->size, dbt2->data, dbt2->size);	
+	}
+
+	return retvalue;
+}
