@@ -84,17 +84,9 @@ typedef void (*MicrofeedDatabaseIndexFunction)(void* key, const size_t key_size,
 typedef int (*MicrofeedDatabaseCompareFunction)(const void* key1, const size_t key1_size, const void* key2, const size_t key2_size);
 
 MicrofeedDatabaseEnvironment* microfeed_database_environment_new(const char* name, const char* directory);
-void microfeed_database_environment_free(MicrofeedDatabaseEnvironment* database_environment);
-MicrofeedDatabaseEnvironment* microfeed_database_environment_ref(MicrofeedDatabaseEnvironment* database_environment);
-void microfeed_database_environment_unref(MicrofeedDatabaseEnvironment* database_environment);
-
-MicrofeedDatabase* microfeed_database_environment_get_database(MicrofeedDatabaseEnvironment* database_environment, const char* name, MicrofeedDatabaseCompareFunction);
-MicrofeedDatabaseIndex* microfeed_database_get_index(MicrofeedDatabase* database, const char* name, MicrofeedDatabaseIndexFunction index_function);
-
-void microfeed_database_free(MicrofeedDatabase* database);
-MicrofeedDatabase* microfeed_database_ref(MicrofeedDatabase* database);
-void microfeed_database_unref(MicrofeedDatabase* database);
+MicrofeedDatabase* microfeed_database_environment_get_database(MicrofeedDatabaseEnvironment* database_environment, const char* name, MicrofeedDatabaseCompareFunction compare_function, int create);
 
+MicrofeedDatabaseIndex* microfeed_database_get_index(MicrofeedDatabase* database, const char* name, MicrofeedDatabaseIndexFunction index_function, MicrofeedDatabaseCompareFunction compare_function);
 const char* microfeed_database_get_name(MicrofeedDatabase* database);
 int microfeed_database_get_data(MicrofeedDatabase* database, const void* key, size_t key_size, void** data, size_t* data_size);
 MicrofeedDatabaseIterator* microfeed_database_iterate(MicrofeedDatabase* database, const void* start_key, size_t start_key_size, int backwards);
