@@ -70,7 +70,7 @@ MicrofeedJson* microfeed_json_new_object(void) {
 	MicrofeedJson* json;
 	
 	json = microfeed_memory_allocate(MicrofeedJson);
-	json->members = microfeed_store_new_sorted((MicrofeedStoreCompareKeysFunction)strcmp, (MicrofeedStoreGetKeyFunction)member_get_name);
+	json->members = microfeed_store_new_sorted_data((MicrofeedStoreCompareKeysFunction)strcmp, (MicrofeedStoreGetKeyFunction)member_get_name);
 	
 	return json;
 }
@@ -79,7 +79,7 @@ MicrofeedJson* microfeed_json_new_array(void) {
 	MicrofeedJson* json;
 	
 	json = microfeed_memory_allocate(MicrofeedJson);
-	json->members = microfeed_store_new_unsorted((MicrofeedStoreCompareKeysFunction)strcmp, (MicrofeedStoreGetKeyFunction)member_get_name);
+	json->members = microfeed_store_new_unsorted_data((MicrofeedStoreCompareKeysFunction)strcmp, (MicrofeedStoreGetKeyFunction)member_get_name);
 	
 	return json;
 }
