@@ -49,6 +49,14 @@ typedef struct {
 	MicrofeedStore* publishers_by_object_path;
 } Provider;
 
+typedef struct _DataStoredCallback DataStoredCallback;
+struct _DataStoredCallback {
+	DataStoredCallback* next;
+	DataStoredCallback* previous;
+	MicrofeedSubscriberDataCallback callback;
+	void* user_data;
+};
+
 typedef struct {
 	unsigned int reference_count;
 	Provider* provider;
@@ -56,6 +64,7 @@ typedef struct {
 	char* object_path;
 	time_t last_activity;
 	MicrofeedStore* feeds;
+	DataStoredCallback* data_stored_callbacks;
 } Publisher;
 
 typedef struct _Subscription Subscription;
@@ -76,7 +85,6 @@ typedef struct {
 	int republishing : 1;
 } Feed;
 
-
 typedef struct {
 	const char* name;
 	void (*callback)(MicrofeedSubscriber* subscriber, Publisher* publisher, DBusMessage* message);
@@ -124,24 +132,24 @@ static void signal_feed_update_started(MicrofeedSubscriber* subscriber, Publishe
 static void signal_feed_update_ended(MicrofeedSubscriber* subscriber, Publisher* publisher, DBusMessage* message);
 static void signal_feed_republishing_started(MicrofeedSubscriber* subscriber, Publisher* publisher, DBusMessage* message);
 static void signal_feed_republishing_ended(MicrofeedSubscriber* subscriber, Publisher* publisher, DBusMessage* message);
-static void signal_item_data(MicrofeedSubscriber* subscriber, Publisher* publisher, DBusMessage* message);
 static void signal_item_added(MicrofeedSubscriber* subscriber, Publisher* publisher, DBusMessage* message);
 static void signal_item_changed(MicrofeedSubscriber* subscriber, Publisher* publisher, DBusMessage* message);
 static void signal_item_republished(MicrofeedSubscriber* subscriber, Publisher* publisher, DBusMessage* message);
 static void signal_item_removed(MicrofeedSubscriber* subscriber, Publisher* publisher, DBusMessage* message);
 static void signal_item_status_changed(MicrofeedSubscriber* subscriber, Publisher* publisher, DBusMessage* message);
+static void signal_data_stored(MicrofeedSubscriber* subscriber, Publisher* publisher, DBusMessage* message);
 
 static SignalCallback signal_callbacks[] = {
 	{ MICROFEED_SIGNAL_NAME_FEED_UPDATE_STARTED, signal_feed_update_started },
 	{ MICROFEED_SIGNAL_NAME_FEED_UPDATE_ENDED, signal_feed_update_ended },
 	{ MICROFEED_SIGNAL_NAME_FEED_REPUBLISHING_STARTED, signal_feed_republishing_started },
 	{ MICROFEED_SIGNAL_NAME_FEED_REPUBLISHING_ENDED, signal_feed_republishing_ended },
-	{ MICROFEED_SIGNAL_NAME_ITEM_DATA, signal_item_data },
 	{ MICROFEED_SIGNAL_NAME_ITEM_ADDED, signal_item_added },
 	{ MICROFEED_SIGNAL_NAME_ITEM_CHANGED, signal_item_changed },
 	{ MICROFEED_SIGNAL_NAME_ITEM_REPUBLISHED, signal_item_republished },
 	{ MICROFEED_SIGNAL_NAME_ITEM_REMOVED, signal_item_removed },
 	{ MICROFEED_SIGNAL_NAME_ITEM_STATUS_CHANGED, signal_item_status_changed },
+	{ MICROFEED_SIGNAL_NAME_DATA_STORED, signal_data_stored },
 	{ NULL, NULL }
 };
 
@@ -394,15 +402,15 @@ int microfeed_subscriber_republish_items(MicrofeedSubscriber* subscriber, const
 	return retvalue;
 }
 
-int microfeed_subscriber_send_item_data(MicrofeedSubscriber* subscriber, const char* publisher_identifier, const char* uri, const char* uid, MicrofeedSubscriberErrorCallback callback, void* user_data) {
+int microfeed_subscriber_store_data(MicrofeedSubscriber* subscriber, const char* publisher_identifier, const char* url, MicrofeedSubscriberErrorCallback callback, void* user_data) {
 	int retvalue = 0;
 	Publisher* publisher;
 	DBusMessage* message;
 	
-	if ((publisher = get_publisher(subscriber, publisher_identifier, NULL, uid, callback, user_data))) {
-		message = dbus_message_new_method_call(publisher->provider->bus_name, publisher->object_path, MICROFEED_DBUS_INTERFACE_PUBLISHER, MICROFEED_METHOD_NAME_SEND_ITEM_DATA);
-		dbus_message_append_args(message, DBUS_TYPE_STRING, &uri, DBUS_TYPE_STRING, &uid, DBUS_TYPE_INVALID);
-		call_publisher_method(publisher, NULL, uid, message, callback, user_data);
+	if ((publisher = get_publisher(subscriber, publisher_identifier, url, NULL, callback, user_data))) {
+		message = dbus_message_new_method_call(publisher->provider->bus_name, publisher->object_path, MICROFEED_DBUS_INTERFACE_PUBLISHER, MICROFEED_METHOD_NAME_STORE_DATA);
+		dbus_message_append_args(message, DBUS_TYPE_STRING, &url, DBUS_TYPE_INVALID);
+		call_publisher_method(publisher, url, NULL, message, callback, user_data);
 		dbus_message_unref(message);
 		retvalue = 1;
 	}
@@ -410,6 +418,51 @@ int microfeed_subscriber_send_item_data(MicrofeedSubscriber* subscriber, const c
 	return retvalue;
 }
 
+void microfeed_subscriber_add_data_stored_callback(MicrofeedSubscriber* subscriber, const char* publisher_identifier, MicrofeedSubscriberDataCallback callback, void* user_data) {
+	Publisher* publisher;
+	DataStoredCallback* data_stored_callback;
+	char buffer[1024];
+	
+	if ((publisher = get_publisher(subscriber, publisher_identifier, NULL, NULL, NULL, NULL))) {
+		data_stored_callback = microfeed_memory_allocate(DataStoredCallback);
+		data_stored_callback->callback = callback;
+		data_stored_callback->user_data = user_data;
+		data_stored_callback->next = publisher->data_stored_callbacks;
+		publisher->data_stored_callbacks = data_stored_callback;
+
+		snprintf(buffer, 1024, "type='signal',path='%s',interface='%s',member='%s'", publisher->object_path, MICROFEED_DBUS_INTERFACE_PUBLISHER, MICROFEED_SIGNAL_NAME_DATA_STORED);
+		dbus_bus_add_match(subscriber->connection, buffer, NULL);
+	}
+}
+
+void microfeed_subscriber_remove_data_stored_callback(MicrofeedSubscriber* subscriber, const char* publisher_identifier, MicrofeedSubscriberDataCallback callback, void* user_data) {
+	Publisher* publisher;
+	DataStoredCallback* data_stored_callback;
+	char buffer[1024];
+	
+	if ((publisher = get_publisher(subscriber, publisher_identifier, NULL, NULL, NULL, NULL))) {
+		for (data_stored_callback = publisher->data_stored_callbacks; data_stored_callback; data_stored_callback = data_stored_callback->next) {
+			if (data_stored_callback->callback == callback && data_stored_callback->user_data == user_data) {
+				if (data_stored_callback->next) {
+					data_stored_callback->next->previous = data_stored_callback->previous;
+				}
+				if (data_stored_callback->previous) {
+					data_stored_callback->previous->next = data_stored_callback->next;
+				} else {
+					publisher->data_stored_callbacks = data_stored_callback->next;
+				}
+
+				microfeed_memory_free(data_stored_callback);
+
+				snprintf(buffer, 1024, "type='signal',path='%s',interface='%s',member='%s'", publisher->object_path, MICROFEED_DBUS_INTERFACE_PUBLISHER, MICROFEED_SIGNAL_NAME_DATA_STORED);
+				dbus_bus_remove_match(subscriber->connection, buffer, NULL);
+
+				break;
+			}
+		}
+	}
+}
+
 int microfeed_subscriber_create_publisher(MicrofeedSubscriber* subscriber, const char* publisher_identifier, MicrofeedSubscriberErrorCallback callback, void* user_data) {
 	int retvalue = 0;
 	Publisher* publisher;
@@ -477,19 +530,15 @@ int microfeed_subscriber_unmark_item(MicrofeedSubscriber* subscriber, const char
 	return retvalue;	
 }
 
-int microfeed_subscriber_read_items(MicrofeedSubscriber* subscriber, const char* publisher_identifier, const char* uri, const char* start_uid, const char* end_uid, MicrofeedSubscriberReplyCallback callback, void* user_data) {
+int microfeed_subscriber_read_item(MicrofeedSubscriber* subscriber, const char* publisher_identifier, const char* uri, const char* uid, MicrofeedSubscriberReplyCallback callback, void* user_data) {
 	int retvalue = 0;
 	Feed* feed;
 	DBusMessage* message;
-	const char* start_uid_dbus;
-	const char* end_uid_dbus;
 
-	if ((feed = get_feed(subscriber, publisher_identifier, uri, start_uid, callback, user_data))) {
-		message = dbus_message_new_method_call(feed->publisher->provider->bus_name, feed->publisher->object_path, MICROFEED_DBUS_INTERFACE_PUBLISHER, MICROFEED_METHOD_NAME_READ_ITEMS);
-		start_uid_dbus = (start_uid ? start_uid : "");
-		end_uid_dbus = (end_uid ? end_uid : "");
-		dbus_message_append_args(message, DBUS_TYPE_STRING, &uri, DBUS_TYPE_STRING, &start_uid_dbus, DBUS_TYPE_STRING, &end_uid_dbus, DBUS_TYPE_INVALID);
-		call_publisher_method(feed->publisher, feed->uri, start_uid, message, callback, user_data);
+	if ((feed = get_feed(subscriber, publisher_identifier, uri, uid, callback, user_data))) {
+		message = dbus_message_new_method_call(feed->publisher->provider->bus_name, feed->publisher->object_path, MICROFEED_DBUS_INTERFACE_PUBLISHER, MICROFEED_METHOD_NAME_READ_ITEM);
+		dbus_message_append_args(message, DBUS_TYPE_STRING, &uri, DBUS_TYPE_STRING, &uid, DBUS_TYPE_INVALID);
+		call_publisher_method(feed->publisher, feed->uri, uid, message, callback, user_data);
 		dbus_message_unref(message);
 		retvalue = 1;
 	}
@@ -497,7 +546,6 @@ int microfeed_subscriber_read_items(MicrofeedSubscriber* subscriber, const char*
 	return retvalue;
 }
 
-
 MicrofeedConfiguration* microfeed_subscriber_get_configuration(MicrofeedSubscriber* subscriber) {
 	
 	return subscriber->configuration;
@@ -810,36 +858,6 @@ static void signal_feed_republishing_ended(MicrofeedSubscriber* subscriber, Publ
 	}
 }
 
-static void signal_item_data(MicrofeedSubscriber* subscriber, Publisher* publisher, DBusMessage* message) {
-	DBusMessageIter iter;
-	char* uri;
-	Feed* feed;
-	char* uid;
-	DBusMessageIter subiter;
-	char* data;
-	int length;
-	Subscription* s;
-		
-	if (dbus_message_iter_init(message, &iter) && dbus_message_iter_get_arg_type(&iter) == DBUS_TYPE_STRING) {
-		dbus_message_iter_get_basic(&iter, &uri);
-		if ((feed = microfeed_store_get(publisher->feeds, uri, Feed)) &&
-		    dbus_message_iter_next(&iter) && dbus_message_iter_get_arg_type(&iter) == DBUS_TYPE_STRING) {
-			dbus_message_iter_get_basic(&iter, &uid);
-			if (dbus_message_iter_next(&iter) && dbus_message_iter_get_arg_type(&iter) == DBUS_TYPE_ARRAY &&
-			    dbus_message_iter_get_element_type(&iter) == DBUS_TYPE_BYTE) {
-				dbus_message_iter_recurse(&iter, &subiter);
-				dbus_message_iter_get_fixed_array(&subiter, &data, &length);
-	
-				for (s = feed->subscriptions; s; s = s->next) {
-					if (s->callbacks->item_data_received) {
-						s->callbacks->item_data_received(subscriber, publisher->identifier, feed->uri, uid, data, length, s->user_data);
-					}
-				}
-			}
-		}
-	}
-}
-
 static void signal_item_added(MicrofeedSubscriber* subscriber, Publisher* publisher, DBusMessage* message) {
 	Feed* feed;
 	MicrofeedItem* item;
@@ -896,7 +914,7 @@ static void signal_item_removed(MicrofeedSubscriber* subscriber, Publisher* publ
 	if (dbus_message_get_args(message, &error, DBUS_TYPE_STRING, &uri, DBUS_TYPE_STRING, &uid, DBUS_TYPE_INVALID) &&
 	    (feed = microfeed_store_get(publisher->feeds, uri, Feed))) {
 		for (s = feed->subscriptions; s; s = s->next) {
-			if (s->callbacks->item_republished) {
+			if (s->callbacks->item_removed) {
 				s->callbacks->item_removed(subscriber, publisher->identifier, feed->uri, uid, s->user_data);
 			}
 		}
@@ -922,6 +940,23 @@ static void signal_item_status_changed(MicrofeedSubscriber* subscriber, Publishe
 	}
 }
 
+static void signal_data_stored(MicrofeedSubscriber* subscriber, Publisher* publisher, DBusMessage* message) {
+	DBusError error;
+	char* url;
+	char* file;
+	char* path = NULL;
+	DataStoredCallback* data_stored_callback;
+
+	dbus_error_init(&error);
+	if (dbus_message_get_args(message, &error, DBUS_TYPE_STRING, &url, DBUS_TYPE_STRING, &file, DBUS_TYPE_INVALID) && publisher->data_stored_callbacks) {
+		path = microfeed_util_string_concatenate(microfeed_configuration_get_publisher_directory(subscriber->configuration, publisher->identifier), "/", publisher->identifier, "/data/", file, NULL);
+		for (data_stored_callback = publisher->data_stored_callbacks; data_stored_callback; data_stored_callback = data_stored_callback->next) {
+			data_stored_callback->callback(subscriber, publisher->identifier, url, path, data_stored_callback->user_data);
+		}
+		free(path);	
+	}
+}
+
 static Publisher* get_publisher(MicrofeedSubscriber* subscriber, const char* publisher_identifier, const char* uri, const char* uid, MicrofeedSubscriberErrorCallback callback, void* user_data) {
 	Publisher* publisher;
 	const char* bus_name;
@@ -1133,6 +1168,7 @@ static DBusHandlerResult object_message(DBusConnection* connection, DBusMessage*
 	MicrofeedStoreIterator* feed_iterator;
 	Feed* feed;
 	int i;
+	const char* path;
 	Provider* provider;
 	Publisher* publisher;
 	DBusMessage* reply;
@@ -1173,8 +1209,9 @@ static DBusHandlerResult object_message(DBusConnection* connection, DBusMessage*
 		microfeed_store_iterator_free(publisher_iterator);
 	}
 
-	if ((provider = microfeed_store_get(subscriber->providers_by_unique_connection_name, dbus_message_get_sender(message), Provider)) &&
-	    (publisher = microfeed_store_get(provider->publishers_by_object_path, dbus_message_get_path(message), Publisher))) {
+	if ((path = dbus_message_get_path(message)) &&
+	    (provider = microfeed_store_get(subscriber->providers_by_unique_connection_name, dbus_message_get_sender(message), Provider)) &&
+	    (publisher = microfeed_store_get(provider->publishers_by_object_path, path, Publisher))) {
 		publisher->last_activity = time(NULL);
 		if (dbus_message_is_method_call(message, MICROFEED_DBUS_INTERFACE_SUBSCRIBER, MICROFEED_METHOD_NAME_PING)) {
 			reply = dbus_message_new_method_return(message);
