@@ -39,18 +39,22 @@
  */
 typedef struct _MicrofeedSubscriber MicrofeedSubscriber;
 
-typedef void* MicrofeedPublisherHandle;
 
 /** @} */
 
 #include <microfeed-common/microfeedstore.h>
 #include <microfeed-common/microfeeditem.h>
 #include <microfeed-common/microfeedconfiguration.h>
+#include <microfeed-common/microfeedprotocol.h>
 
 /**
  * @addtogroup MicrofeedSubscriber
  * @{
  */
+
+#define MICROFEED_DBUS_OBJECT_PATH_PREFIX_SUBSCRIBER MICROFEED_DBUS_OBJECT_PATH_PREFIX "subscriber/"
+#define MICROFEED_SUBSCRIBER_DBUS_METHOD_CALL_TIMEOUT 600000
+
 typedef void (*MicrofeedSubscriberReplyCallback)(MicrofeedSubscriber* subscriber, const char* publisher, const char* uri, const char* uid, const char* error_name, const char* error_message, void* user_data);
 typedef void (*MicrofeedSubscriberErrorCallback)(MicrofeedSubscriber* subscriber, const char* publisher, const char* uri, const char* uid, const char* error_name, const char* error_message, void* user_data);
 typedef void (*MicrofeedSubscriberFeedCallback)(MicrofeedSubscriber* subscriber, const char* publisher, const char* uri, void* user_data);
@@ -58,8 +62,6 @@ typedef void (*MicrofeedSubscriberItemCallback)(MicrofeedSubscriber* subscriber,
 typedef void (*MicrofeedSubscriberItemRemovedCallback)(MicrofeedSubscriber* subscriber, const char* publisher, const char* uri, const char* uid, void* user_data);
 typedef void (*MicrofeedSubscriberStatusCallback)(MicrofeedSubscriber* subscriber, const char* publisher, const char* uri, const char* uid, const MicrofeedItemStatus status, void* user_data);
 typedef void (*MicrofeedSubscriberItemDataCallback)(MicrofeedSubscriber* subscriber, const char* publisher, const char* uri, const char* uid, const void* data, const size_t length, void* user_data);
-typedef void (*MicrofeedSubscriberConfiguredSubscribeCallback)(MicrofeedSubscriber* subscriber, const char* publisher, void* user_data);
-typedef void (*MicrofeedSubscriberConfiguredUnsubscribeCallback)(MicrofeedSubscriber* subscriber, const char* publisher, const char* uri, void* user_data);
 
 typedef struct {
 	MicrofeedSubscriberErrorCallback error_occured;
@@ -75,23 +77,30 @@ typedef struct {
 	MicrofeedSubscriberItemDataCallback item_data_received;
 } MicrofeedSubscriberCallbacks;
 
-MicrofeedSubscriber* microfeed_subscriber_new(const char* identifier, const char* object_path, DBusConnection* connection);
+typedef void (*MicrofeedSubscriberConfiguredSubscribeCallback)(MicrofeedSubscriber* subscriber, const char* publisher, void* callback_data);
+typedef void (*MicrofeedSubscriberConfiguredUnsubscribeCallback)(MicrofeedSubscriber* subscriber, const char* publisher, const char* uri, MicrofeedSubscriberCallbacks* callbacks, void* callbacks_user_data, void* user_data);
+
+MicrofeedSubscriber* microfeed_subscriber_new(const char* identifier, DBusConnection* connection);
 void microfeed_subscriber_free(MicrofeedSubscriber* subscriber);
 
 const char* microfeed_subscriber_get_identifier(MicrofeedSubscriber* subscriber);
+
+int microfeed_subscriber_subscribe_feed(MicrofeedSubscriber* subscriber, const char* publisher, const char* uri, MicrofeedSubscriberCallbacks* callbacks, void* callbacks_user_data, MicrofeedSubscriberReplyCallback callback, void* user_data);
+int microfeed_subscriber_unsubscribe_feed(MicrofeedSubscriber* subscriber, const char* publisher, const char* uri, MicrofeedSubscriberCallbacks* callbacks, void* callbacks_user_data, MicrofeedSubscriberReplyCallback callback, void* user_data);
+
 int microfeed_subscriber_add_item(MicrofeedSubscriber* subscriber, const char* publisher, const char* uri, MicrofeedItem* item, MicrofeedSubscriberReplyCallback callback, void* user_data);
 int microfeed_subscriber_modify_item(MicrofeedSubscriber* subscriber, const char* publisher, const char* uri, MicrofeedItem* item, MicrofeedSubscriberReplyCallback callback, void* user_data);
 int microfeed_subscriber_remove_item(MicrofeedSubscriber* subscriber, const char* publisher, const char* uri, const char* uid, MicrofeedSubscriberReplyCallback callback, void* user_data);
-int microfeed_subscriber_subscribe_feed(MicrofeedSubscriber* subscriber, const char* publisher, const char* uri, MicrofeedSubscriberCallbacks* callbacks, MicrofeedSubscriberReplyCallback callback, void* user_data);
-int microfeed_subscriber_unsubscribe_feed(MicrofeedSubscriber* subscriber, const char* publisher, const char* uri, MicrofeedSubscriberReplyCallback callback, void* user_data);
 int microfeed_subscriber_update_feed(MicrofeedSubscriber* subscriber, const char* publisher, const char* uri, MicrofeedSubscriberReplyCallback callback, void* user_data);
 int microfeed_subscriber_republish_items(MicrofeedSubscriber* subscriber, const char* publisher, const char* uri, const char* start_uid, const char* end_uid, unsigned int max_count, MicrofeedSubscriberReplyCallback callback, void* user_data);
-int microfeed_subscriber_create_publisher(MicrofeedSubscriber* subscriber, const char* publisher, MicrofeedSubscriberReplyCallback callback, void* user_data);
-int microfeed_subscriber_destroy_publisher(MicrofeedSubscriber* subscriber, const char* publisher, MicrofeedSubscriberReplyCallback callback, void* user_data);
-int microfeed_subscriber_send_item_data(MicrofeedSubscriber* subscriber, const char* publisher_identifier, const char* uri, const char* uid, MicrofeedSubscriberReplyCallback callback, void* user_data);
+int microfeed_subscriber_get_item_data(MicrofeedSubscriber* subscriber, const char* publisher_identifier, const char* uri, const char* uid, MicrofeedSubscriberReplyCallback callback, void* user_data);
 int microfeed_subscriber_mark_item(MicrofeedSubscriber* subscriber, const char* publisher_identifier, const char* uri, const char* uid, MicrofeedSubscriberReplyCallback callback, void* user_data);
 int microfeed_subscriber_unmark_item(MicrofeedSubscriber* subscriber, const char* publisher_identifier, const char* uri, const char* uid, MicrofeedSubscriberReplyCallback callback, void* user_data);
 int microfeed_subscriber_read_items(MicrofeedSubscriber* subscriber, const char* publisher_identifier, const char* uri, const char* start_uid, const char* end_uid, MicrofeedSubscriberReplyCallback callback, void* user_data);
+
+int microfeed_subscriber_create_publisher(MicrofeedSubscriber* subscriber, const char* publisher, MicrofeedSubscriberReplyCallback callback, void* user_data);
+int microfeed_subscriber_destroy_publisher(MicrofeedSubscriber* subscriber, const char* publisher, MicrofeedSubscriberReplyCallback callback, void* user_data);
+
 int microfeed_subscriber_handle_configured_subscriptions(MicrofeedSubscriber* subscriber, MicrofeedSubscriberConfiguredSubscribeCallback configured_subscribe, MicrofeedSubscriberConfiguredUnsubscribeCallback configured_unsubscribe, void* user_data);
 
 /**
